﻿using GrapeCity.Documents.Excel;
using GrapeCity.Documents.Excel.Drawing;
using Org.BouncyCastle.Asn1.X509;
using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.Crypto.Generators;
using Org.BouncyCastle.Crypto.Prng;
using Org.BouncyCastle.Math;
using Org.BouncyCastle.Security;
using Org.BouncyCastle.X509;
using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Security.Cryptography.X509Certificates;

namespace DigitalSignaturesDemo_GcExcel
{
    class Program
    {
        private static string FinancialReport = "DigitalSignaturesDemo_GcExcel.Resources.FinancialReport.xlsx";
        private static string SignedFinancialReport = "DigitalSignaturesDemo_GcExcel.Resources.SignedFinancialReport.xlsx";
        enum SignatureType { SignExcel = 1, SignExistingExcel}

        // maps user input to signature type
        private static Dictionary<int, SignatureType> _userInput = new Dictionary<int, SignatureType>()
        {
            {1, SignatureType.SignExcel },
            {2, SignatureType.SignExistingExcel },
        };
        static void Main(string[] args)
        {
            Console.WriteLine("Demo started\n**********************************");
            string userInput = string.Empty;
            int selectedOperation = 0;
            do
            {
                do
                {
                    Console.WriteLine("What would you like to do? Please enter a number corresponding to the operation you want to perform:");
                    Console.WriteLine("1. Sign Excel file");
                    Console.WriteLine("2. Sign existing Excel file");

                    Console.Write("Enter the number here and press Enter: ");
                    userInput = Console.ReadLine();

                    bool isInputNumeric = int.TryParse(userInput, out selectedOperation);
                    if (isInputNumeric && (selectedOperation > 0 && selectedOperation < 3))
                        break;
                    else
                        Console.WriteLine("Incorrect value entered!! Please enter value in the range 1 to 2 followed by Enter key\n----------------------------------");
                } while (true);

                SignDigitally(_userInput[selectedOperation]);

                Console.WriteLine("Operation successful!!\n**********************************");
                string moreOperationsFlag = "n";
                do
                {
                    Console.Write("Do you want to perform more operations? Please enter y (Yes) or n (No) here and press Enter: ");
                    moreOperationsFlag = Console.ReadLine();
                    if (moreOperationsFlag.ToLower() == "y")
                        break;
                    else if (moreOperationsFlag.ToLower() == "n")
                    {
                        Console.Write("Thank you for using GcExcel, closing the console");
                        Environment.Exit(0);
                    }
                    else
                        Console.WriteLine("Incorrect value entered!! Please enter either 'y' or 'n' followed by Enter key\n----------------------------------");
                } while (true);
            } while (true);
        }
        static void SignDigitally(SignatureType signatureType)
        {
            var workbook = new Workbook();

            var assembly = typeof(DigitalSignaturesDemo_GcExcel.Program).GetTypeInfo().Assembly;
            Stream fileStream = assembly.GetManifestResourceStream(signatureType == SignatureType.SignExcel ? FinancialReport : SignedFinancialReport);

            switch (signatureType)
            {
                case SignatureType.SignExcel:
                    workbook.Open(fileStream);
                    break;
                case SignatureType.SignExistingExcel:
                    XlsxOpenOptions openOption = new XlsxOpenOptions { DigitalSignatureOnly = true };
                    workbook.Open(fileStream);
                    break;
            }
            ISignature signature = workbook.Signatures.AddNonVisibleSignature();

            X509Certificate2 cert = new X509Certificate2("Resources/GPCTAdmin.pfx", "grapecity@123");
            SignatureDetails details = new SignatureDetails
            {
                Address1 = "GrapeCity India Pvt Ltd",
                Address2 = "A-15, Sector 62",
                SignatureComments = "Final",
                City = "Noida",
                StateOrProvince = "Uttar Pradesh",
                PostalCode = "201307",
                CountryName = "India",
                ClaimedRole = "Software Engineer",
                CommitmentTypeDescription = "Approved",
                CommitmentTypeQualifier = "Final"
            };

            signature.Sign(cert, details);
            switch (signatureType)
            {
                case SignatureType.SignExcel:
                    workbook.Save("SignedWorkbook.xlsx");
                    break;
                case SignatureType.SignExistingExcel:
                    workbook.Save("FinalSignedWorkbook.xlsx");
                    break;
            }
        }
    }
}
